/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.heuristic.service;

import org.springframework.stereotype.Service;
import ru.ulstu.extractor.heuristic.api.StructuralUnitIdentifier;
import ru.ulstu.extractor.heuristic.model.StructuralUnit;
import ru.ulstu.extractor.service.GitRepositoryService;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.function.Predicate;

@Service
public class StructuralUnitService {
    private final GitRepositoryService gitRepositoryService;
    private final DirectoryService directoryService;
    private final List<StructuralUnitIdentifier> structuralUnitIdentifiers;

    public StructuralUnitService(GitRepositoryService gitRepositoryService,
                                 DirectoryService directoryService,
                                 List<StructuralUnitIdentifier> structuralUnitIdentifiers) {
        this.gitRepositoryService = gitRepositoryService;
        this.directoryService = directoryService;
        this.structuralUnitIdentifiers = structuralUnitIdentifiers;
    }

    public List<StructuralUnit> getEntities(String repositoryUrl) throws IOException {
        File rootPath = gitRepositoryService.getProjectDirectoryFile(repositoryUrl);
        List<File> projectFiles = directoryService.getFilesRecursively(rootPath);
        List<File> rootProjectFiles = directoryService.getDirectoryFiles(rootPath.toPath());
        return getStructuralUnitIdentifier(
                structuralUnitIdentifier -> structuralUnitIdentifier.canAppliedToProject(
                        rootPath.getPath(),
                        projectFiles,
                        rootProjectFiles)
        ).getEntityClasses(rootPath.getPath(), projectFiles, rootProjectFiles);
    }

    public boolean containsEntity(File projectFile) {
        return getStructuralUnitIdentifier(
                structuralUnitIdentifier -> structuralUnitIdentifier.canAppliedToFile(projectFile)
        ).isEntityClass(projectFile);
    }

    private StructuralUnitIdentifier getStructuralUnitIdentifier(Predicate<StructuralUnitIdentifier> predicate) {
        return structuralUnitIdentifiers.stream()
                .filter(predicate)
                .findAny()
                .orElseThrow(() -> new RuntimeException("Identifier not found"));
    }
}
